# $Id: Config.pm 629 2006-01-26 21:30:37Z guillomovitch $
package Youri::Config;

=head1 NAME

Youri::Config - AppConfig extension with YAML support

=head1 SYNOPSIS

    use Youri::Config;

    my $config = Youri::Config->new();

    # read a configuration file
    $config->file("application.conf");

    # get structured values
    my $foo = $config->get('foo');
    my %bar = $config->varlist('bar');

=head1 DESCRIPTION

This class extends AppConfig with ability to use YAML for defining arbitrary
data structures directly from configuration files: any value with first line
having YAML header will get automatically expanded.

Here is an exemple configuration file:

    [updates]
    class    = Youri::Check::Check::Updates
    grabbers = <<EOF
    --- #YAML:1.0
    debian:
        class: Youri::Check::Check::Updates::Debian
        aliases:
            fuse: ~
    cpan:
        class: Youri::Check::Check::Updates::CPAN
    fedora:
        class: Youri::Check::Check::Updates::Fedora
    gentoo:
        class: Youri::Check::Check::Updates::Gentoo
    freshmeat:
        class: Youri::Check::Check::Updates::Freshmeat
        aliases:
            fuse: fuse-emulator
    EOF

=head1 SEE ALSO

AppConfig, YAML

=cut

use strict;
use warnings;
use base qw/AppConfig/;

use Carp;
use YAML;

sub import {
    my $pkg = caller();
    eval <<EOF;
package $pkg;
AppConfig->import(':argcount', ':expand');
EOF
}

=head2 get_section($id)

Simple wrapper around $config->varlist(), throwing a warning if section I<$id> doesn't exists.

=cut

sub get_section {
    my ($self, $id) = @_;
    croak "Not a class method" unless ref $self;

    my %values = $self->varlist('^' . $id . '_', 1);

    carp "No such section $id" unless %values;

    return %values;
}

sub varlist { 
    my ($self, $criteria, $strip) = @_;

    my %values = $self->SUPER::varlist($criteria, $strip);

    foreach my $value (values %values) {
        $value = _yamlize($value);
    }

    return %values;
}

sub get {
    my ($self, $variable) = @_;
    croak "Not a class method" unless ref $self;

    return _yamlize($self->SUPER::get($variable));
}

sub _yamlize {
    my ($value) = @_;

    if ($value =~ /^--- #YAML:1.0/) {
        eval {
            $value = Load($value . "\n");
        };
        $value = undef if $@;
    }

    return $value;
}

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2002-2006, YOURI project

This program is free software; you can redistribute it and/or modify it under the same terms as Perl itself.

=cut

1;
